import { useEffect, useRef } from "react";
/**
 * `useInterval` is a custom hook that runs a function at a specified interval.
 *
 * @see Docs https://ui.ducor.net/hooks/use-interval
 */
const useInterval = (callback, delay) => {
    const savedCallback = useRef(callback);
    useEffect(() => {
        savedCallback.current = callback;
    }, [callback]);
    useEffect(() => {
        if (delay === null)
            return;
        const intervalId = setInterval(() => savedCallback.current(), delay);
        return () => clearInterval(intervalId);
    }, [delay]);
};
export default useInterval;
