/**
 * A robust timeout hook with manual control and auto-invocation options
 * @template T - Type of parameters passed to the callback
 * @param callback - Function to execute after delay
 * @param delay - Delay in milliseconds (null to disable)
 * @param options - Configuration options
 * @returns Object with control methods
 */
declare function useTimeout<T extends unknown[]>(callback: (...args: T) => void, delay: number | null, options?: {
    autoInvoke?: boolean;
    autoInvokeParams?: T;
}): {
    start: (...args: T) => void;
    clear: () => void;
    restart: (...args: T) => void;
    isActive: () => boolean;
};
export default useTimeout;
