import { useEffect } from "react";
import { useCallbackRef } from "../use-outside-click";
/**
 * `useWindowEvent` is a custom hook that assigns an event listener to `window`.
 *
 * @see Docs https://ui.ducor.net/hooks/use-window-event
 */
const useWindowEvent = (event, handler, options) => {
    const listener = useCallbackRef(handler);
    useEffect(() => {
        window.addEventListener(event, listener, options);
        return () => {
            window.removeEventListener(event, listener, options);
        };
    }, [event, listener, options]);
};
export default useWindowEvent;
